#include <MaskedScalerFilter.h>
#include <highgui.h>

// Dominik Schmidt (2008)
// schmidtd@comp.lancs.ac.uk
// http://eis.comp.lancs.ac.uk/~dominik/cms

MaskedScalerFilter::MaskedScalerFilter(char* s) : Filter(s) {
	scalerMap = NULL;
	rectifyMap = NULL;
}

MaskedScalerFilter::~MaskedScalerFilter() {
	if(scalerMap != NULL)
		cvReleaseImage(&scalerMap);
	if(rectifyMap != NULL)
		cvReleaseImage(&rectifyMap);
}

void MaskedScalerFilter::getParameters(ParameterMap& pMap) {
	pMap[std::string("scalerFilename")] = toString(scalerMapFile);
	pMap[std::string("rectifyFilename")] = toString(rectifyMapFile);
}

void MaskedScalerFilter::setParameter(const char *name, const char *value) {
	if (strcmp(name, "scalerFilename") == 0) {
		scalerMapFile = new char[255];
		sprintf(scalerMapFile, "%s", value);
	} else if (strcmp(name, "rectifyFilename") == 0) {
		rectifyMapFile = new char[255];
		sprintf(rectifyMapFile, "%s", value);
	}
}

void MaskedScalerFilter::showOutput(bool value, int windowx, int windowy) {
	Filter::showOutput(value, windowx, windowy);	
}

void MaskedScalerFilter::kernel() {
	if(!destination) {
        destination = cvCreateImage(cvSize(source->width,source->height), source->depth, 1);
        destination->origin = source->origin;
    }	
	if(scalerMap == NULL) {
		scalerMap = cvLoadImage(scalerMapFile, 0);
		if(!scalerMap) {
			printf("Unable to load filter mask from %s\n", scalerMapFile);
			scalerMap = cvCreateImage(cvGetSize(source), source->depth, source->nChannels);			
		}		
	}
	if(rectifyMap == NULL) {
		rectifyMap = cvLoadImage(rectifyMapFile, 0);
		if(!rectifyMap) {
			printf("Unable to load filter mask from %s\n", rectifyMapFile);
			rectifyMap = cvCreateImage(cvGetSize(source), source->depth, source->nChannels);			
		}		
	}

	int value;
	for(int y = 0; y < source->height; y++) {		
		uchar* ptrDes = (uchar*) (
			destination->imageData + y * destination->widthStep
		);
		uchar* ptrSrc = (uchar*) (
			source->imageData + y * source->widthStep
		);
		uchar* ptrMas = (uchar*) (
			scalerMap->imageData + y * scalerMap->widthStep
		);
		uchar* ptrRec = (uchar*) (
			rectifyMap->imageData + y * rectifyMap->widthStep
		);
		for(int x = 0; x < source->width; x++) {	
			// scaler
			value = (int) (ptrSrc[x] * ptrSrc[x]  * ((float) ptrMas[x] / 128.0f));						

			// rectify
			if(value <= ptrRec[x])
				ptrDes[x] = 0;
			else {				
				if(value > 255)
					value = 255;
				ptrDes[x] = value;				
			}
		}
	}
}